@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-"""Copyright (C) 2009-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+"""Copyright (C) 2009-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,15 +24,14 @@ Read the user manual for a description of the interface to this scoring engine
 import types
 from hashlib import md5
 
-from qt import QVariant
-
-from common import Internal, Debug, english  # pylint: disable=redefined-builtin
-from common import unicodeString, StrMixin, nativeString, nativeStringArgs
-from log import m18n, m18nc, m18nE, logException, logDebug
+from common import Internal, Debug
+from common import StrMixin
+from log import logException, logDebug
+from mi18n import i18n, i18nc, i18nE, i18ncE, english
 from query import Query
 
 
-class Score(object):
+class Score(StrMixin):
 
     """holds all parts contributing to a score. It has two use cases:
     1. for defining what a rules does: either points or doubles or limits, holding never more than one unit
@@ -51,10 +50,10 @@ class Score(object):
         self.doubles = type(self.doubles)(doubles)
         self.limits = type(self.limits)(limits)
 
-    unitNames = {m18nE(
-                 'points'): 0,
-                 m18nE('doubles'): 50,
-                 m18nE('limits'): 9999}
+    unitNames = {i18nE(
+        'points'): 0,
+                 i18ncE('kajongg', 'doubles'): 50,
+                 i18ncE('kajongg', 'limits'): 9999}
 
     def clear(self):
         """set all to 0"""
@@ -63,8 +62,6 @@ class Score(object):
     def change(self, unitName, value):
         """sets value for unitName. If changed, return True"""
         oldValue = self.__getattribute__(unitName)
-        if isinstance(value, QVariant):
-            value = value.toString()
         newValue = type(oldValue)(value)
         if newValue == oldValue:
             return False, None
@@ -89,19 +86,19 @@ class Score(object):
             parts.append('limits=%f' % self.limits)
         return ' '.join(parts)
 
-    def __repr__(self):
-        return 'Score(%s)' % str(self)
-
-    def contentStr(self):
+    def i18nStr(self):
         """make score readable for humans, i18n"""
         parts = []
         if self.points:
-            parts.append(m18nc('Kajongg', '%1 points', self.points))
+            parts.append(i18nc('Kajongg', '%1 points', self.points))
         if self.doubles:
-            parts.append(m18nc('Kajongg', '%1 doubles', self.doubles))
+            parts.append(i18nc('Kajongg', '%1 doubles', self.doubles))
         if self.limits:
-            parts.append(m18nc('Kajongg', '%1 limits', self.limits))
-        return u' '.join(parts)
+            limits = str(self.limits)
+            if limits.endswith('.0'):
+                limits = limits[-2:]
+            parts.append(i18nc('Kajongg', '%1 limits', limits))
+        return ' '.join(parts)
 
     def __eq__(self, other):
         """ == comparison """
@@ -220,11 +217,9 @@ class RuleList(list):
                 rule.key(), self[rule.key()].definition, rule.name, rule.definition))
         self[rule.key()] = rule
 
-    def createRule(self, name, definition='', **kwargs):
+    def createRule(self, name: str, definition: str='', **kwargs):
         """shortcut for simpler definition of predefined rulesets"""
-        name = unicodeString(name)
-        definition = unicodeString(definition)
-        defParts = definition.split(u'||')
+        defParts = definition.split('||')
         rule = None
         description = kwargs.get('description', '')
         for cls in [IntRule, BoolRule, StrRule]:
@@ -238,7 +233,7 @@ class RuleList(list):
         if not rule:
             if 'parameter' in kwargs:
                 del kwargs['parameter']
-            ruleType = type(str(ruleKey(name)) + 'Rule', (Rule, ), {})
+            ruleType = type(ruleKey(name) + 'Rule', (Rule, ), {})
             rule = ruleType(name, definition, **kwargs)
             if defParts[0] == 'FCallingHand':
                 parts1 = defParts[1].split('=')
@@ -246,14 +241,14 @@ class RuleList(list):
                 ruleClassName = parts1[1] + 'Rule'
                 if ruleClassName not in RuleBase.ruleClasses:
                     logDebug(
-                        u'we want %s, definition:%s' %
+                        'we want %s, definition:%s' %
                         (ruleClassName, definition))
-                    logDebug(u'we have %s' % RuleBase.ruleClasses.keys())
+                    logDebug('we have %s' % RuleBase.ruleClasses.keys())
                 ruleType.limitHand = RuleBase.ruleClasses[ruleClassName]
         self.add(rule)
 
 
-class UsedRule(object):
+class UsedRule(StrMixin):
 
     """use this in scoring, never change class Rule.
     If the rule has been used for a meld, pass it"""
@@ -268,11 +263,8 @@ class UsedRule(object):
             result += ' ' + str(self.meld)
         return result
 
-    def __repr__(self):
-        return 'UsedRule(%s)' % str(self)
 
-
-class Ruleset(object):
+class Ruleset:
 
     """holds a full set of rules: meldRules,handRules,winnerRules.
 
@@ -349,19 +341,19 @@ class Ruleset(object):
         self.doublingMeldRules = []
         self.doublingHandRules = []
         self.standardMJRule = None
-        self.meldRules = RuleList(1, m18n('Meld Rules'),
-                                  m18n('Meld rules are applied to single melds independent of the rest of the hand'))
-        self.handRules = RuleList(2, m18n('Hand Rules'),
-                                  m18n('Hand rules are applied to the entire hand, for all players'))
-        self.winnerRules = RuleList(3, m18n('Winner Rules'),
-                                    m18n('Winner rules are applied to the entire hand but only for the winner'))
-        self.loserRules = RuleList(33, m18n('Loser Rules'),
-                                   m18n('Loser rules are applied to the entire hand but only for non-winners'))
-        self.mjRules = RuleList(4, m18n('Mah Jongg Rules'),
-                                m18n('Only hands matching a Mah Jongg rule can win'))
-        self.parameterRules = RuleList(999, m18nc('kajongg', 'Options'),
-                                       m18n('Here we have several special game related options'))
-        self.penaltyRules = RuleList(9999, m18n('Penalties'), m18n(
+        self.meldRules = RuleList(1, i18n('Meld Rules'),
+                                  i18n('Meld rules are applied to single melds independent of the rest of the hand'))
+        self.handRules = RuleList(2, i18n('Hand Rules'),
+                                  i18n('Hand rules are applied to the entire hand, for all players'))
+        self.winnerRules = RuleList(3, i18n('Winner Rules'),
+                                    i18n('Winner rules are applied to the entire hand but only for the winner'))
+        self.loserRules = RuleList(33, i18n('Loser Rules'),
+                                   i18n('Loser rules are applied to the entire hand but only for non-winners'))
+        self.mjRules = RuleList(4, i18n('Mah Jongg Rules'),
+                                i18n('Only hands matching a Mah Jongg rule can win'))
+        self.parameterRules = RuleList(999, i18nc('kajongg', 'Options'),
+                                       i18n('Here we have several special game related options'))
+        self.penaltyRules = RuleList(9999, i18n('Penalties'), i18n(
             """Penalties are applied manually by the user. They are only used for scoring games.
 When playing against the computer or over the net, Kajongg will never let you get
 into a situation where you have to pay a penalty"""))
@@ -425,18 +417,14 @@ into a situation where you have to pay a penalty"""))
                 "select id,hash,name,description from ruleset where id=?", (self.name,))
         elif isinstance(self.name, list):
             # we got the rules over the wire
-            self.rawRules = (nativeStringArgs(x) for x in self.name[1:])
+            self.rawRules = self.name[1:]
             (self.rulesetId, self.__hash, self.name,
              self.description) = self.name[0]
-            self.__hash = nativeString(self.__hash)
             self.load()
                       # load raw rules at once, rules from db only when needed
             return
         else:
-            query = Query(
-                "select id,hash,name,description from ruleset where hash=?",
-                (nativeString(self.name),
-                 ))
+            query = Query("select id,hash,name,description from ruleset where hash=?", (self.name,))
         if len(query.records):
             (self.rulesetId, self.__hash, self.name,
              self.description) = query.records[0]
@@ -487,7 +475,7 @@ into a situation where you have to pay a penalty"""))
         """returns a Query object with loaded ruleset"""
         return Query(
             "select ruleset, list, position, name, definition, points, doubles, limits, parameter from rule "
-                "where ruleset=? order by list,position", (self.rulesetId,))
+            "where ruleset=? order by list,position", (self.rulesetId,))
 
     def toList(self):
         """returns entire ruleset encoded in a string"""
@@ -562,9 +550,9 @@ into a situation where you have to pay a penalty"""))
             copyNr = 1
             while self.nameExists(newName):
                 copyStr = ' ' + str(copyNr) if copyNr > 1 else ''
-                newName = m18nc(
+                newName = i18nc(
                     'Ruleset._newKey:%1 is empty or space plus number',
-                    'Copy%1 of %2', copyStr, m18n(self.name))
+                    'Copy%1 of %2', copyStr, i18n(self.name))
                 copyNr += 1
         return newId, newName
 
@@ -611,21 +599,14 @@ into a situation where you have to pay a penalty"""))
             Query("DELETE FROM rule WHERE ruleset=?", (self.rulesetId,))
             Query("DELETE FROM ruleset WHERE id=?", (self.rulesetId,))
 
-    @staticmethod
-    def ruleKey(rule):
-        """needed for sorting the rules"""
-        return rule.__unicode__()
-
     def __computeHash(self):
         """compute the hash for this ruleset using all rules but not name and
         description of the ruleset"""
         self.load()
         result = md5()
-        for rule in sorted(self.allRules, key=Ruleset.ruleKey):
+        for rule in sorted(self.allRules, key=Rule.__str__):
             result.update(rule.hashStr().encode('utf-8'))
         self.__hash = result.hexdigest()
-        # TODO: warum geht fuer HC.newTable ein u'fdfads' als hash vom
-        # Py2-Client zum Server?
 
     def ruleRecord(self, rule):
         """returns the rule as tuple, prepared for use by sql. The first three
@@ -727,7 +708,7 @@ into a situation where you have to pay a penalty"""))
             qData = Query(
                 "select name from ruleset where id=?",
                 (lastUsedId,
-                 )).records
+                )).records
             if qData:
                 lastUsed = qData[0][0]
                 for idx, ruleset in enumerate(result):
@@ -761,12 +742,12 @@ class RuleBase(StrMixin):
     options = {}
     ruleClasses = {}
 
-    def __init__(self, name, definition, description):
-        self.__name = unicodeString(name)
-        self.definition = unicodeString(definition)
-        self.description = unicodeString(description)
+    def __init__(self, name: str, definition: str, description: str):
         self.hasSelectable = False
         self.ruleClasses[self.__class__.__name__] = self.__class__
+        self.__name = name
+        self.definition = definition
+        self.description = description
 
     @property
     def name(self):
@@ -782,11 +763,11 @@ class RuleBase(StrMixin):
         all that is needed to hash this rule
 
         @return: The unique hash string
-        @rtype: unicode
+        @rtype: str
         """
-        return u''
+        return ''
 
-    def __unicode__(self):
+    def __str__(self):
         return self.hashStr()
 
 
@@ -933,20 +914,20 @@ class Rule(RuleBase):
         payees = int(self.options.get('payees', 1))
         if not 2 <= payers + payees <= 4:
             logException(
-                m18nc(
+                i18nc(
                     '%1 can be a sentence', '%4 have impossible values %2/%3 in rule "%1"',
-                                  self.name, payers, payees, 'payers/payees'))
+                    self.name, payers, payees, 'payers/payees'))
 
     def explain(self, meld):
         """use this rule for scoring"""
-        return '%s: %s' % (m18n(
+        return '%s: %s' % (i18n(
             self.explainTemplate if self.explainTemplate else self.name).format(
                 group=meld[0].groupName() if meld else '',
                 value=meld[0].valueName() if meld else '',
                 meldType=meld.typeName() if meld else '',
                 meldName=meld.name() if meld else '',
-                tileName=meld[0].name() if meld else ''
-        ).replace('&', '').replace('  ', ' ').strip(), self.score.contentStr())
+                tileName=meld[0].name() if meld else '').replace(
+                    '&', '').replace('  ', ' ').strip(), self.score.i18nStr())
 
     def hashStr(self):
         """
@@ -954,13 +935,13 @@ class Rule(RuleBase):
         database congestion low.
 
         @return: The unique hash string
-        @rtype: unicode
+        @rtype: str
         """
-        return u'%s: %s %s' % (self.name, self.definition, self.score)
+        return '%s: %s %s' % (self.name, self.definition, self.score)
 
-    def contentStr(self):
+    def i18nStr(self):
         """returns a human readable string with the content"""
-        return self.score.contentStr()
+        return self.score.i18nStr()
 
     @staticmethod
     def exclusive():
@@ -969,7 +950,7 @@ class Rule(RuleBase):
 
     def hasNonValueAction(self):
         """Rule has a special action not changing the score directly"""
-        return bool(any(x not in ['lastsource', 'declaration'] for x in self.options))
+        return bool(any(x not in ['lastsource', 'announcements'] for x in self.options))
 
 
 class ParameterRule(RuleBase):
@@ -999,12 +980,12 @@ class ParameterRule(RuleBase):
         database congestion low.
 
         @return: The unique hash string
-        @rtype: unicode
+        @rtype: str
         """
-        result = u'%s: %s %s' % (self.name, self.definition, self.parameter)
+        result = '%s: %s %s' % (self.name, self.definition, self.parameter)
         return result
 
-    def contentStr(self):
+    def i18nStr(self):
         """returns a human readable string with the content"""
         return str(self.parameter)
 
@@ -1029,9 +1010,9 @@ class IntRule(ParameterRule):
     def validate(self):
         """is the rule valid?"""
         if self.parameter < self.minimum:
-            return m18nc(
+            return i18nc(
                 'wrong value for rule', '%1: %2 is too small, minimal value is %3',
-                   m18n(self.name), self.parName, self.minimum)
+                i18n(self.name), self.parName, self.minimum)
 
 
 class BoolRule(ParameterRule):
@@ -1050,7 +1031,7 @@ class BoolRule(ParameterRule):
 
 class StrRule(ParameterRule):
 
-    """for str parameters. Duck typing with Rule"""
+    """for str parameters. Duck typing with Rule. Currently not used."""
     prefix = 'str'
 
     def __init__(self, name, definition, description, parameter):
