@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 import weakref
 
+from common import StrMixin, Debug
 from tile import Tile, elements
 
 
@@ -29,7 +30,7 @@ class WallEmpty(Exception):
     pass
 
 
-class KongBox(object):
+class KongBox:
 
     """a non-ui kong box"""
 
@@ -56,7 +57,7 @@ class KongBox(object):
         return len(self._tiles)
 
 
-class Wall(object):
+class Wall(StrMixin):
 
     """represents the wall with four sides. self.wall[] indexes them
     counter clockwise, 0..3. 0 is bottom.
@@ -68,8 +69,11 @@ class Wall(object):
     def __init__(self, game):
         """init and position the wall"""
         self._game = weakref.ref(game)  # avoid cycles for garbage collection
+        wallSize = int(Debug.wallSize)
+        if not wallSize:
+            wallSize = elements.count(game.ruleset)
         self.tiles = [self.tileClass(Tile.unknown)
-                      for _ in range(elements.count(game.ruleset))]
+                      for _ in range(wallSize)]
         self.living = None
         self.kongBox = self.kongBoxClass()
         assert len(self.tiles) % 8 == 0
@@ -117,7 +121,7 @@ class Wall(object):
         """to be done only for UIWall"""
         pass
 
-    def decorate(self):
+    def decorate4(self, deferredResult=None): # pylint: disable=unused-argument
         """virtual: show player info on the wall"""
 
     def hide(self):
@@ -141,3 +145,12 @@ class Wall(object):
                 boxTiles[pair * 2]] + \
                 boxTiles[pair * 2 + 2:]
         self.kongBox.fill(boxTiles)
+
+    @staticmethod
+    def name():
+        """name for debug messages"""
+        return '4sided wall'
+
+    def __str__(self):
+        """for debugging"""
+        return self.name()
